//******************************************
// file			: MQ6801_8BIT_TC0x.c
// version		: V1.1 2021/02/24
// brief		: 8-bit TC0 related functions
// note			: 8-bit TC0 related functions are gathered in this subroutine
//******************************************

#include "main.h"




//*********************************************************************
// name			: TC01_Timer_Init
// brief		: TC01 8-bit timer mode initialization (set period to 100us)
// note			:
//*********************************************************************
void TC01_Timer_Init(void)
{
//====== enable function ======
    POFFCR0_TC001EN = 1;                // this register must be enable first and then the related registers settings will work

//====== parameter setting ======
    T01MOD_TCM      = 0;                // set operation mode to 8-bit timer/event counter modes
    T01MOD_TCK      = 5;                // fcgck/4 = 1MHz = 1us
    T01MOD_DBE      = 1;                // enable the double buffer
    
    T01REG          = 100;              // set counter value to 100
}


//*********************************************************************
// name			: TC01_PPG_Init
// brief		: TC01 8-bit PPG mode initialization (set period to 100us and duty cycle to 50%)
// note			:
//*********************************************************************
void TC01_PPG_Init(void)
{
//====== pin configuration ======
    P7DR_P1         = 1;                // set P71 to output high (suggest output level as the same as the value of T0xMOD_TFF)
    P7CR_P1         = 1;                // set P71 as output pin

//====== enable function ======
    POFFCR0_TC001EN = 1;                // this register must be enable first and then the related registers settings will work

//====== parameter setting ======
    T01MOD_TFF      = 1;                // set initial state of function pin to high

    T01MOD_TCM      = 3;                // set operation mode to 8-bit PPG mode
    T01MOD_TCK      = 5;                // fcgck/4 = 1MHz = 1us
    T01MOD_DBE      = 1;                // enable the double buffer

    T01REG          = 100;              // set PPG period = 100us   (PPG period=T00REG*T00MOD_TCK)
    T01PWM          = 50;               // set PPG duty cycle = 50/100 = 50%
	
//	***set function pins after function initialization
    P7FC_P1         = 1;                // set P71 as PPG function pin	
}


//*********************************************************************
// name			: TC01_PWM_Init
// brief		: TC01 8-bit PWM mode initialization (set period to 128us and duty cycle to 50%)
// note			:
//*********************************************************************
void TC01_PWM_Init(void)
{
//====== pin configuration ======
    P7DR_P1         = 1;                // set P71 to output high (suggest output level as the same as the value of T0xMOD_TFF)
    P7CR_P1         = 1;                // set P71 as output pin

//====== enable function ======
    POFFCR0_TC001EN = 1;                // this register must be enable first and then the related registers settings will work

//====== parameter setting ======
    T01MOD_TFF      = 1;                // set initial state of function pin to high

    T01MOD_TCM      = 2;                // set operation mode to 8-bit PWM mode
    T01MOD_TCK      = 5;                // fcgck/4 = 1MHz = 1us
    T01MOD_DBE      = 1;                // enable the double buffer

// ***the pulse-width modulated pulses with a resolution of 7 bits are output in the 8-bit PWM mode. T00REG is invaild.
//  128 * 1us = 128us
    T01PWM          = 128;              // set PWM duty cycle = (128/2)/128 = 50%

//	***set function pins after function initialization
    P7FC_P1         = 1;                // set P71 as PWM function pin
}


//*********************************************************************
// name			: TC01_SetDuty
// brief		: set duty cycle
// note			: duty cycle : T01PWM/T01REG      (8-bit PPG mode)
//              : (T01PWM/2) / 128   (8-bit PWM mode)
//*********************************************************************
void TC01_SetDuty(uchar duty)
{
    T01PWM          = duty;
}

//*********************************************************************
// name			: TC01_SetPeriod
// brief		: set period
// note			: 
//*********************************************************************
void TC01_SetPeriod(uchar period)
{
    T01REG          = period;
}

//*********************************************************************
// name			: TC01_Start
// brief		: start TC01
// note			:
//*********************************************************************
void TC01_Start(void)
{
    T001CR_T01RUN   = 1;                // start to run TC01
}


//*********************************************************************
// name			: TC01_Stop
// brief		: stop TC01
// note			:
//*********************************************************************
void TC01_Stop(void)
{
    T001CR_T01RUN   = 0;                // stop running TC01
}


//*********************************************************************
// name			: TC01_IntEnable
// brief		: enable TC01 interrupt
// note			: after execution, Interrupt Master Allowed Flag (IMF) will be enabled
//*********************************************************************
void TC01_IntEnable(void)
{
    DI;                                 // disable all maskable interrupts // EIRL_IMF=0
    IL_INTTC01      = 0;                // clear interrupt latch
    EI_INTTC01      = 1;                // set interrupt enable flag
    EI;                                 // enable all maskable interrupts // EIRL_IMF=1
}




//*********************************************************************
// name			: TC01_IntDisable
// brief		: disable TC01 interrupt
// note			: after execution, Interrupt Master Allowed Flag (IMF) will be enabled
//*********************************************************************
void TC01_IntDisable(void)
{
    DI;                                 // disable all maskable interrupts // EIRL_IMF=0
    IL_INTTC01      = 0;                // clear interrupt latch
    EI_INTTC01      = 0;                // clear interrupt enable flag
    EI;                                 // enable all maskable interrupts // EIRL_IMF=1
}




//*********************************************************************
// name			: IntTC01
// brief		: interrupt service routine
// note			:
//*********************************************************************
void __interrupt IntTC01(void)
{
    P9DR_P0 = !P9DR_P0;					// reverse P90 output level
    NOP;                                // no operation
}





