//******************************************
// file			: MQ6801_ADC.h
// version		: V1.1 2021/02/24
// brief		: ADC header file
// note			:
//******************************************
#ifndef _MQ6801_ADC_H__
#define _MQ6801_ADC_H__



#define	ad_test_times       16

/******************
 * 1st Argument
 * ****************/
#define     ADC_Single      0x1 // single mode
#define     ADC_Repeat      0x3 // repeat mode
/******************
 * 2nd Argument
 * ****************/
#define     AIN_DIS         0x0 // analog input disable
#define     AIN_EN          0x1 // analog input enable
/******************
 * 3rd Argument
 * ****************/
#define     AIN0            0x0 // set analog input channel to AIN0
#define     AIN1            0x1 // set analog input channel to AIN1
#define     AIN2            0x2 // set analog input channel to AIN2
#define     AIN3            0x3 // set analog input channel to AIN3
#define     AIN4            0x4 // set analog input channel to AIN4
#define     AIN6            0x6 // set analog input channel to AIN6
#define     VDD4_1          0x9 // set analog input channel to internal 1/4 VDD input
#define     AIN_GND         0xc // set analog input channel to internal ground

/******************
 * 4th Argument
 * ****************/
// fcgck=16M... Conv256CLK(16us) ~ Conv1024CLK( 64us)
// fcgck= 8M... Conv128CLK(16us) ~ Conv1024CLK(128us)
// fcgck= 4M... Conv64CLK (16us) ~ Conv512CLK (128us)
// fcgck= 2M... Conv32CLK (16us) ~ Conv256CLK (128us)
#define     Conv32CLK       0x0 // set conversion times to 32 fcgck
#define     Conv64CLK       0x1 // set conversion times to 64 fcgck
#define     Conv128CLK      0x2 // set conversion times to 128 fcgck
#define     Conv256CLK      0x3 // set conversion times to 256 fcgck
#define     Conv512CLK      0x4 // set conversion times to 512 fcgck
#define     Conv1024CLK     0x5 // set conversion times to 1024 fcgck
                                // (*conversion times must be between 12.8 us to 128 us)
/******************
 * 5th Argument
 * ****************/
#define     Vref2           0x0 // set ADC reference voltage to internal 2V

#define     VDDVref         0x3 // set ADC reference voltage to internal VDD
#define     EXTVref         0x4 // set ADC reference voltage to external voltage (AIN3)
								// when AIN3 is used as the external voltage input, AIN3 cannot be used to analog input





extern uint  vref_l, vref_h, iReal_Vref2V, ADC_Code_READ;
extern ulong VrefReal2048;


void ADC_Init(uchar SAIN);
void ADC_Start(void);
void ADC_Stop(void);
void ADC_IntEnable(void);
void ADC_IntDisable(void);

uint GetVref(void);
uint ADC_single_detect(uchar SAIN);
uint ADC_repeat_detect(uchar SAIN);

void __interrupt IntADC(void);






#endif








