//******************************************
// file			: main.c
// version		: V1.0
// brief		: main program
// note			: example of flash memory operation
// 		      make sure that the chip does not power off or reset before the FLASH function is completely
//******************************************
#ifndef IO_MEM
#define IO_MEM
#endif

#include "main.h"

//==============================================================================
// 	variable
//==============================================================================
unsigned char ReadData,ReadBuf[10],ret[10];

main()
{
    /* ---------------------- system initialization ------------------------ */
    System_Init();


    /* ---------------------- xxxx function initialization ------------------- */

//==============================================================================
//	Flash_SectionErase(address)
//  address	:0x7e80~0x7eff,0x8000~0xffff
//  retval  :0:erase success  1:erase fail 3:invalid address parameter
//  the flash memory area to be erased is specified by address
// 
//  note 	: when erase, IMF=0 will be set temporarily, CPU will not accept "maskable interrupt" signal for the time being
//   
//==============================================================================	
	ret[0]=Flash_SectionErase(0x7e80);//erase 0x7e80~0x7eff
	ret[1]=Flash_SectionErase(0xff00);//erase 0xff00 ~0xff7f	
	ret[2]=Flash_SectionErase(0xfe02);//erase 0xfe00 ~0xfe7f	

//==============================================================================
//	Flash_WriteByte(address,data)
//  address	:0x7e80~0x7eff,0x8000~0xffff
//  data	:0x00~0xff
//  retval  : 0:write success  1:write fail<difference from expected value after writing>  2:write fail<not 0xFF before writing>
//            3:invalid address parameter
//  note 	: when erase, IMF=0 will be set temporarily, CPU will not accept "maskable interrupt" signal for the time being
//==============================================================================	
	ret[3]=Flash_WriteByte(0x7e80,0x55);
	ret[4]=Flash_WriteByte(0xff00,0xaa);
	ret[5]=Flash_WriteByte(0xfe01,0x51);
	ret[6]=Flash_WriteByte(0xfe02,0x52);
	ret[7]=Flash_WriteByte(0xfe03,0x53);
	
//==============================================================================
//	Flash_ReadByte(address)
//  address	:0x7e80~0x7eff,0x8000~0xffff
//  retval  :0x00~0xff
//==============================================================================		
	ReadData = Flash_ReadByte(0x7e80);
	ReadData = Flash_ReadByte(0xff00);
	ReadData = Flash_ReadByte(0xfe01);	
			
//==============================================================================
//	Flash_ReadSequence(address,data array,data length)
//  address	:0x7e80~0x7eff,0x8000~0xffff
//  data array	:array of unsigned char type
//  data length	:read byte counts
// 
//  note: the watchdog will be cleared in the function, if the watchdog has special settings (limited to clear the first 1/4, 1/2, 3/4 overflow time), it may generate a watchdog interrupt
//==============================================================================			
	Flash_ReadSequence(0xfe01,ReadBuf,3);
	
	
//---dummy loop---//
	while(1)
    {
        CLR_WDT;                        // clear the watchdog timer

        NOP;                            // no operation
        NOP;                            // no operation
        NOP;                            // no operation
    }
//  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

}









/*---------------------- interrupt service routine ------------------------*/
void __interrupt OnlyReti(void)
{
    NOP;                                // no operation
}

void __interrupt_n OnlyRetn(void)
{
    NOP;                                // no operation
}

void __interrupt_n IntWDT(void)
{
    NOP;                                // no operation
}

void __interrupt_n IntSWI(void)
{
    unsigned char *p_coming_opcode = ((*(unsigned short *)(__SP + (6 * 2) + 1)) - 1);
    unsigned char r_index;
    static const unsigned char a_invalid_opcode[] = {  /* invalid opcode */
            0x01,0x02,0x03,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0xf8,0xff };		

    for( r_index = 0; r_index < ( sizeof(a_invalid_opcode)/sizeof(a_invalid_opcode[0])); r_index++ )
    {
        if ( *p_coming_opcode == a_invalid_opcode[r_index] )
            SYSCR2=0x10;	//system clock reset
    }

    // adjust address for return from IntSWI // '
    (*(unsigned short *)(__SP + (6 * 2) + 1))--;
		
    // cost 6 bytes for fix stack //
    __asm("dec	wa");
    __asm("dec	bc");
    __asm("dec	de");
    __asm("dec	hl");
    __asm("dec	ix");
    __asm("dec	iy");	
}      

/* ------------------- end of interrupt service routine ------------------ */

/* -------------------- MQ6812 interrupt vector ------------------ */
#pragma section const INT_VECTOR1 
void * const IntTbl1[] = {  
    OnlyReti,           // 0xffc4 : IntTC05
    OnlyReti,           // 0xffc6 : IntTC04
    OnlyReti,           // 0xffc8 : Reserved
    OnlyReti,           // 0xffca : Reserved
    OnlyReti,           // 0xffcc : IntTC03
    OnlyReti,           // 0xffce : IntTC02
    OnlyReti,           // 0xffd0 : IntTXD1
    OnlyReti,           // 0xffd2 : IntRXD1
    OnlyReti,           // 0xffd4 : Reserved
    OnlyReti,           // 0xffd6 : Int4
    OnlyReti,           // 0xffd8 : Int3
    OnlyReti,           // 0xffda : Int2
    OnlyReti,           // 0xffdc : Reserved
    OnlyReti,           // 0xffde : Reserved
    OnlyReti,           // 0xffe0 : IntSBI
    OnlyReti,           // 0xffe2 : IntTCA0
    OnlyReti,           // 0xffe4 : IntTC01
    OnlyReti,           // 0xffe6 : IntTC00
    OnlyReti,           // 0xffe8 : IntRTC
    OnlyReti,           // 0xffea : IntADC
    OnlyReti,           // 0xffec : IntVLTD
    OnlyReti,           // 0xffee : Reserved
    OnlyReti,           // 0xfff0 : Reserved
    OnlyReti,           // 0xfff2 : IntSIO0
    OnlyReti,           // 0xfff4 : IntTBT
    OnlyReti,           // 0xfff6 : IntWUC
    IntWDT,             // 0xfff8 : IntWDT
    (void *)0xffff,     // 0xfffa : Reserved
    IntSWI,             // 0xfffc : IntSWI / INTUNDEF
    STARTUP             // 0xfffe : RESET	
};  
#pragma section const  			
/*------------------- end of interrupt vector ------------------------*/






//*********************************************************************
// name			: System_Init(void)
// brief		: IO, CGCR and P10/P40/P41 initialization during debugging
// note			:
//*********************************************************************
void System_Init(void)
{

//=============================================================================
//  Set CGCR
//=============================================================================
//  ============================================
//  clock gear control registerCGCR                                  fc   =  16MHz
//  ============================================
    CGCR_FCGCKSEL = 0x00;           // (system clock) fcgck = fc/4 =  4Mhz(default)
//  CGCR_FCGCKSEL = 0x01;           // (system clock) fcgck = fc/2 =  8Mhz
//  CGCR_FCGCKSEL = 0x02;           // (system clock) fcgck = fc   =  16Mhz
//  CGCR_FCGCKSEL = 0x03;           // (system clock) fcgck = fc/8 =  2Mhz
//  ============================================
//  *** do not change the value of CGCR<FCGCKSEL> register in low-speed mode (SLOW1/SLOW2)
//  ============================================

#ifdef _DEBUG   //...  _DEBUG: for emulation of ev board
//------------------------------------------------------------------
//  (default): when emulation, set P10 to Reset pin to avoid affecting the emulation and program functions
//  SET_P10_RESET_PIN;
//------------------------------------------------------------------
    FSCTRL_FSSEL = 0;                   // if the program has changed this bit, you need to manually initialize and clear it to the default value of 0 in the emulation mode 
//------------------------------------------------------------------
#else           //...  in general power on and run the program (in non-emulation mode), set P10 to GPIO pin
//------------------------------------------------------------------
//-- set P10 to GPIO pin ------
//   SET_P10_IO_PIN;
//------------------------------------------------------------------
	
//------------------------------------------------------------------

#endif



//-- pin configuration initialization (debug pin configuration)------
#ifdef _DEBUG   //...  _DEBUG: for emulation of ev board
//------------------------------------------------------------------
//  the simulation uses P10, P40 and P41 into Debug mode, which is set as an input during simulation
//  to avoid affecting the emulation and program functions
    P1DR = 0x00;
    P1CR = set8bit(11111110);           // set P10 as input pin
    P4DR = 0x00;
    P4CR = set8bit(11111100);           // set P40 and P41 as input pins
//------------------------------------------------------------------
#else           //...  in general power on and run the program (in non-emulation mode)
//------------------------------------------------------------------
    P1DR = 0x00;
    P1CR = 0xff;
    P4DR = 0x00;
    P4CR = 0xff;
//------------------------------------------------------------------
#endif

//-- pin configuration initialization ------
    P0DR = 0x00;
    P0CR = set8bit(11111100);
//  P1DR = 0x00;
//  P1CR = 0xff;
    P2DR = 0x00;
    P2CR = 0xff;
//  P4DR = 0x00;
//  P4CR = 0xff;    
    P7DR = 0x00;
    P7CR = 0xff;
    P8DR = 0x00;
    P8CR = 0xff;
    P9DR = 0x00;
    P9CR = 0xff;
    CLR_WDT;                            // clear the watchdog timer

}



//*********************************************************************
// name			:
// brief		: time delay function
// note			: ==> t_1ms   = 0;          // time delay unit ms
//          : ==> t_100us = 1;          // time delay unit us (this accuracy is not good)
//          : the example of using a fixed loop with a time delay is as follows:
//          : Delay_Times(t_1ms, 800);  // delay 800ms
//*********************************************************************
void Delay_Times(uchar tBase, uint x)
{
    uint  i, cgcrT;

    uchar cgcrF[4]={ 2, 4,  8, 1 };     // use this array when the high-speed crystal frequency is 8MHz
//  uchar cgcrF[4]={ 4, 8, 16, 2 };     // use this array when the high-speed crystal frequency is 16MHz

//  the time delay function of the system at NORMAL1 mode and NORMAL2 mode
    if( SYSCR2_SYSCK == 0 )
    {
        cgcrT = cgcrF[CGCR_FCGCKSEL];   // 0:fc/4, 1:fc/2, 2:fc/1, 3:fc/8
        switch(tBase)
        {
            case 0:                     // t_1ms
                cgcrT *= 110;
                for(; x>0; x--)
                    for(i=0; i<cgcrT; i++){}
            break;
            case 1:                     // t_100us
                cgcrT *= 11;
                for(; x>0; x--)
                    for(i=0; i<cgcrT; i++){}
            break;
        }
    }
//  the time delay function of the system at SLOW1 mode and SLOW2 mode), which has a large error, is used for reference
    else for(; x>0; x--){}

    CLR_WDT;                            // clear the watchdog timer
}









