//******************************************
// file			: MQ6823_startup.c
// version		: V1.1 2019/03/21MQ6823
// brief		: startup program
// note			: Set fcgck=16M Hz
//******************************************
//******************************************
//******************************************
/*
 * --------------------------------------------------------------------------
 *   HEADER INCLUDE DEFINITIONS
 * --------------------------------------------------------------------------
 */
#include <stdlib.h>
#include "MQ6823_SFR.h"    // declare the detailed names represented by each bit of the special function registers

/*
 * --------------------------------------------------------------------------
 *   MACRO DEFINITIONS
 * --------------------------------------------------------------------------
 */
extern unsigned int _BaseSP;				/* the starting address of the stacking pointer is generally defined as the the last address of the SRAM */
extern unsigned int _RAM_Top;				/* the first address of SRAM */
extern unsigned int _RAM_End;				/* the last address of SRAM */

extern unsigned char _TDataAddr[];			/* for tiny_data initialize, defined in link command file  */
extern unsigned char _TDataOrg[];			/* for tiny_data initialize, defined in link command file  */
extern unsigned int _TDataSize;				/* for tiny_data initialize, defined in link command file  */
extern unsigned char _NDataAddr[];			/* for near_data initialize, defined in link command file  */
extern unsigned char _NDataOrg[];			/* for near_data initialize, defined in link command file  */
extern unsigned int _NDataSize;				/* for near_data initialize, defined in link command file  */

/*
 * --------------------------------------------------------------------------
 *   FUNCTION DECLARATIONS
 * --------------------------------------------------------------------------
 */

/*#**************************************************************************#*/

/*
 ******************************************************************************
 * NAME				:STARTUP
 *-----------------------------------------------------------------------------
 * DESCRIPTION		:Startup routine
 * PARAMETER		:-
 *
 * RETURN VALUE		:-
 *
 ******************************************************************************
 */
void STARTUP( void )
{
//=============================================================================
//  disable interrupt
//=============================================================================
    DI;                                 // disable all maskable interrupts // EIRL_IMF=0


//=============================================================================
//  Set CGCR
//=============================================================================
//  ============================================
//  clock gear control registerCGCR                                  fc   = 16MHz
//  ============================================
//  CGCR_FCGCKSEL = 0x00;           // (system clock) fcgck = fc/4 =  4Mhz(ϵͳĬ)
//  CGCR_FCGCKSEL = 0x01;           // (system clock) fcgck = fc/2 =  8Mhz
    CGCR_FCGCKSEL = 0x02;           // (system clock) fcgck = fc   = 16Mhz
//  CGCR_FCGCKSEL = 0x03;           // (system clock) fcgck = fc/8 =  2Mhz
//  ============================================
//  *** do not change the value of CGCR<FCGCKSEL> register in low-speed mode (SLOW1/SLOW2)
//  ============================================


//=============================================================================
//  initial Registers
//=============================================================================
    __WA = 0x00;
    __BC = 0x00;
    __DE = 0x00;
    __HL = 0x00;
    __IX = 0x00;
    __IY = 0x00;

//=============================================================================
//  initialize stack pointer
//=============================================================================
    __asm(" LD SP,__BaseSP");       //initialize stack pointer

//=============================================================================
//  clear status word
//=============================================================================
    __asm(" LD PSW,0x00");

//=============================================================================
//  clear RAM
//=============================================================================
    __asm(" LD HL,__RAM_Top");
    __asm(" LD BC,__RAM_End - __RAM_Top");
    __asm(" LD A,0x00");
    __asm("CLR_RAM:");
    __asm(" LD (HL),A");
    __asm(" INC HL");
    __asm(" DEC BC");
    __asm(" JRS F,CLR_RAM");        // Clear the RAM

//=============================================================================
//  initialize of tiny_data section
//=============================================================================
    __asm("	LD	IX,__TDataOrg");
    __asm("	LD	IY,__TDataAddr");
    __asm("	LD	A,__TDataSize");
    __asm("	DEC	A");
    __asm("	J	T,non_t_data");
    __asm("move_t_data:");
    __asm("	LD	C,(IX)");
    __asm("	LD	(IY),C");
    __asm("	INC	IX");
    __asm("	INC	IY");
    __asm("	DEC	A");
    __asm("	J	F,move_t_data");
    __asm("non_t_data:");

//=============================================================================
//  initialize of near_data section
//=============================================================================
    __asm("	LD	IX,__NDataOrg");
    __asm("	LD	IY,__NDataAddr");
    __asm("	LD	WA,__NDataSize");
    __asm("	DEC	WA");
    __asm("	J	T,non_n_data");
    __asm("move_n_data:");
    __asm("	LD	C,(IX)");
    __asm("	LD	(IY),C");
    __asm("	INC	IX");
    __asm("	INC	IY");
    __asm("	DEC	WA");
    __asm("	J	F,move_n_data");
    __asm("non_n_data:");

//=============================================================================
//  other system configuration
//=============================================================================
    //System_Config();
	
	
//=============================================================================
//	Set UnPad Pin normal output LOW to avoid leakage 2019-03/18
//=============================================================================

// 	set Low
// 	P22,P25~P27     0001 1011  0x1B
// 	P45~P47         0001 1111  0x1F
// 	P52,P57         0111 1011  0x7B
// 	P77             0111 1111  0x7F

	P2DR    = P2DR & ( 0x1B ); // P22,P25~P27 output low
	P4DR    = P4DR & ( 0x1F ); // P45~P47     output low
	P5DR    = P5DR & ( 0x7B ); // P52,P57	  output low
	P7DR    = P7DR & ( 0x7F ); // P77	      output low
//----------------------------------------------
// 	set Output
// 	P22,P25~P27     1110 0100  0xE4
// 	P45~P47         1110 0000  0xE0
// 	P52,P57         1000 0100  0x84
// 	P77             1000 0000  0x80

	P2CR    = P2CR | ( 0xE4 ); // P22,P25~P27 output
	P4CR    = P4CR | ( 0xE0 ); // P45~P47	  output
	P5CR    = P5CR | ( 0x84 ); // P52,P57     output
	P7CR    = P7CR | ( 0x80 ); // P77	      output

//=============================================================================
//  jump to main
//=============================================================================
	main();
	for (;;) {
		;
	}
}

