//**************************************************************
// file			: main.c
// version		: V1.1 2021/09/13
// brief		: main program
// note			: SIO master transmit/receive example with "SIO_Master" program
//**************************************************************
#ifndef IO_MEM
#define IO_MEM
#endif

#include "main.h"

/* ---------------------- global variables ------------------------ */

/* ---------------------- main program -------------------------- */
void main(){
    /* ---------------------- system initialization ------------------------ */
    SYSTEM_INIT();

    /* ---------------------- function initialization ------------------- */
    pLoop = 0;
    SIO_Init();							//SIO initialization
    SIO0_IntEnable();
	SIO0BUF = 0xff;
	SIO_Start();						//start SIO
	
	
	


//------------------------------------------------------------------------
//  1. this example is for SIO slave transmission (SIO clock is external clock input)
//  2. it can be used to communicate with the SIO_Master example chip
//  3. the basic commands are as follows:
//     3-1. 0x9f master reads Device ID 0xaa 0x68 0x35
//     3-2. 0x03 master reads data from slave
//     3-3. 0x02 master writes data to slave
//------------------------------------------------------------------------
	
//  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    while(1)
    {
        CLR_WDT;                        // clear the watchdog timer

        NOP;                            // no operation
        NOP;                            // no operation
        NOP;                            // no operation
    }
//  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

}


















/*---------------------- interrupt service routine ------------------------*/
void __interrupt OnlyReti(void)
{
    NOP;                                // no operation
}

void __interrupt_n OnlyRetn(void)
{
    NOP;                                // no operation
}

void __interrupt_n IntWDT(void)
{
    NOP;                                // no operation
}

void __interrupt_n IntSWI(void)
{
    unsigned char *p_coming_opcode = ((*(unsigned short *)(__SP + (6 * 2) + 1)) - 1);
    unsigned char r_index;
    static const unsigned char a_invalid_opcode[] = {  /* invalid opcode */
            0x01,0x02,0x03,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0xf8,0xff };		

    for( r_index = 0; r_index < ( sizeof(a_invalid_opcode)/sizeof(a_invalid_opcode[0])); r_index++ )
    {
        if ( *p_coming_opcode == a_invalid_opcode[r_index] )
            SYSCR2 = 0x10; 	//system clock reset
    }

    // adjust address for return from IntSWI // '
    (*(unsigned short *)(__SP + (6 * 2) + 1))--;
		
    // cost 6 bytes for fix stack //
    __asm("dec	wa");
    __asm("dec	bc");
    __asm("dec	de");
    __asm("dec	hl");
    __asm("dec	ix");
    __asm("dec	iy");	
}     
/*-----------------  жϷ ------------------------*/

/*------------------------ interrupt vector ------------------------*/
#pragma section const INT_VECTOR1
void * const IntTbl1[] = {
    OnlyReti,           // 0xffc0 : IntTCC2P
    OnlyReti,           // 0xffc2 : IntTCC2T
    OnlyReti,           // 0xffc4 : IntTCC1P
    OnlyReti,           // 0xffc6 : IntTCC1T
    OnlyReti,           // 0xffc8 : IntTCA1
    OnlyReti,           // 0xffca : IntSIO1
    OnlyReti,           // 0xffcc : IntTCC0P
    OnlyReti,           // 0xffce : IntTCC0T
    OnlyReti,           // 0xffd0 : IntTXD1
    OnlyReti,           // 0xffd2 : IntRXD1
    OnlyReti,           // 0xffd4 : Int5
    OnlyReti,           // 0xffd6 : Int4
    OnlyReti,           // 0xffd8 : Int3 / IntCMP1
    OnlyReti,           // 0xffda : Int2 / IntCMP0 / IntEMG0
    OnlyReti,           // 0xffdc : Int1 / IntCMP2
    OnlyReti,           // 0xffde : Int0 / IntCMP3
    OnlyReti,           // 0xffe0 : IntSBI
    OnlyReti,           // 0xffe2 : IntTCA0
    OnlyReti,           // 0xffe4 : IntTC01
    OnlyReti,           // 0xffe6 : IntTC00
    OnlyReti,           // 0xffe8 : IntRTC
    OnlyReti,           // 0xffea : IntADC
    OnlyReti,           // 0xffec : IntLVD
    OnlyReti,           // 0xffee : IntTXD0
    OnlyReti,           // 0xfff0 : IntRXD0
    IntSIO0,            // 0xfff2 : IntSIO0
    OnlyReti,           // 0xfff4 : IntTBT
    OnlyReti,           // 0xfff6 : IntWUC
    IntWDT,             // 0xfff8 : IntWDT
    (void *)0xffff,     // 0xfffa : Reserved
    IntSWI,             // 0xfffc : IntSWI / INTUNDEF
    STARTUP             // 0xfffe : RESET
};
#pragma section const
/*------------------- end of interrupt vector ------------------------*/

















//*********************************************************************
// name			: SYSTEM_INIT(void)
// brief		: IO, CGCR and P10/P40/P41 initialization during debugging
// note			:
//*********************************************************************
void SYSTEM_INIT(void)
{

//=============================================================================
//  Set CGCR
//=============================================================================
//  ============================================
//  clock gear control registerCGCR                                  fc   = 16MHz
//  ============================================
    CGCR_FCGCKSEL = 0x00;           // (system clock) fcgck = fc/4 =  4Mhz(default)
//  CGCR_FCGCKSEL = 0x01;           // (system clock) fcgck = fc/2 =  8Mhz
//  CGCR_FCGCKSEL = 0x02;           // (system clock) fcgck = fc   = 16Mhz
//  CGCR_FCGCKSEL = 0x03;           // (system clock) fcgck = fc/8 =  2Mhz
//  ============================================
//  *** do not change the value of CGCR<FCGCKSEL> register in low-speed mode (SLOW1/SLOW2)
//  ============================================

#ifdef _DEBUG   //...  _DEBUG: for emulation of ev board
//------------------------------------------------------------------
//  (default): when emulation, set P10 to Reset pin to avoid affecting the emulation and program functions
    SET_P10_to_RESET_PIN;
//------------------------------------------------------------------
    FSCTRL_FSSEL = 0;                   // if the program has changed this bit, you need to manually initialize and clear it to the default value of 0 in the emulation mode 
//------------------------------------------------------------------
#else           //...  in general power on and run the program (in non-emulation mode), set P10 to GPIO pin
//------------------------------------------------------------------
//-- set P10 to GPIO pin ------
    SET_P10_to_IO_PIN;
//------------------------------------------------------------------
#endif



//-- pin configuration initialization (debug pin configuration)------
#ifdef _DEBUG   //...  _DEBUG: for emulation of ev board
//------------------------------------------------------------------
//  the simulation uses P10, P40 and P41 into Debug mode, which is set as an input during simulation
//  to avoid affecting the emulation and program functions
    P1DR = 0x00;
    P1CR = set8bit(11111110);           // set P10 as input pin
    P4DR = 0x00;
    P4CR = set8bit(11111100);           // set P40 and P41 as input pins
//------------------------------------------------------------------
#else           //...  in general power on and run the program (in non-emulation mode)
//------------------------------------------------------------------
    P1DR = 0x00;
    P1CR = 0xff;
    P4DR = 0x00;
    P4CR = 0xff;
//------------------------------------------------------------------
#endif

//-- pin configuration initialization ------
    P0DR = 0x00;
    P0CR = 0xff;
//  P1DR = 0x00;
//  P1CR = 0xff;
    P2DR = 0x00;
    P2CR = 0xff;
//  P4DR = 0x00;
//  P4CR = 0xff;
    P5DR = 0x00;
    P5CR = 0xff;
    P7DR = 0x00;
    P7CR = 0xff;
    P8DR = 0x00;
    P8CR = 0xff;
    P9DR = 0x00;
    P9CR = 0xff;
//  sendout_I2C(IRSTSR);                // IRSTSR: internal factor reset detection status register
    CLR_WDT;                            // clear the watchdog timer




}



//*********************************************************************
// name			: delay_times
// brief		: time delay function
// note			: ==> t_1ms   = 0;          // time delay unit ms
//          : ==> t_100us = 1;          // time delay unit us (this accuracy is not good)
//          : the example of using a fixed loop with a time delay is as follows:
//          : delay_times(t_1ms, 800);  // delay 800ms
//*********************************************************************
void delay_times(uchar tBase, uint x)
{
    uint  i, cgcrT;

//  uchar cgcrF[4]={ 2, 4,  8, 1 };     // use this array when the high-speed crystal frequency is 8MHz
    uchar cgcrF[4]={ 4, 8, 16, 2 };     // use this array when the high-speed crystal frequency is 16MHz

//  the time delay function of the system at NORMAL1 mode and NORMAL2 mode
    if( SYSCR2_SYSCK == 0 )
    {
        cgcrT = cgcrF[CGCR_FCGCKSEL];   // 0:fc/4, 1:fc/2, 2:fc/1, 3:fc/8
        switch(tBase)
        {
            case 0:                     // t_1ms
                cgcrT *= 110;
                for(; x>0; x--)
                    for(i=0; i<cgcrT; i++){}
            break;
            case 1:                     // t_100us
                cgcrT *= 11;
                for(; x>0; x--)
                    for(i=0; i<cgcrT; i++){}
            break;
        }
    }
//  the time delay function of the system at SLOW1 mode and SLOW2 mode), which has a large error, is used for reference
    else for(; x>0; x--){}

    CLR_WDT;                            // clear the watchdog timer
}




//*********************************************************************
// name			: sendout_I2C(uchar data)
// brief		:
// note			: emulate I2C with GPIO
//*********************************************************************
void sendout_I2C(uchar data)
{
    uchar iloop, x, z=10;

    io_SDA_cr = 1;                      // setting output port
    io_SCL_cr = 1;                      // setting output port

//	start condition......
    io_SDA = 1;         for(x=z; x>0; x--){}
    io_SCL = 1;         for(x=z; x>0; x--){}
    io_SDA = 0;         for(x=z; x>0; x--){}
    io_SCL = 0;         for(x=z; x>0; x--){}

//	sent byte data......
    for( iloop=0; iloop<8; iloop++ )
    {
        io_SDA = data >> 7;
        data   = data << 1;
        for(x=z; x>0; x--){}
        io_SCL = 1;     for(x=z; x>0; x--){}
        io_SCL = 0;     for(x=z; x>0; x--){}
    }
    io_SDA = 0;         for(x=z; x>0; x--){}
    io_SCL = 1;         for(x=z; x>0; x--){}
    io_SCL = 0;         for(x=z; x>0; x--){}

    //	stop condition......
    io_SDA = 0;         for(x=z; x>0; x--){}
    io_SCL = 1;         for(x=z; x>0; x--){}
    io_SDA = 1;         for(x=z; x>0; x--){}
}










