//SQ7515_DIV example
//*************************************************************************************
// file			: main.c
// version		: V1.1 2022/06/08
// brief		: main program
// description	: using DIV to perform division
//
//			  when press button 1: a(dividend) = 0x44B17E22, b(divisor) = 0x00000045, if the result is correct, light on LED1
//      	  when press button 2: a(dividend) = 0x00001234, b(divisor) = 0x00005678, if the result is correct, light on LED2
//      	  when press button 3: a(dividend) = 0xFEDCBA98, b(divisor) = 0x01234567, if the result is correct, light on LED3
//            *when the button is released, the LED will turn off and wait for next press
//
// note			: 
//				 [Button Configuration] 
//                button 1 : BTN1_in(P3.0)
//				  button 2 : BTN2_in(P3.1)
//				  button 3 : BTN3_in(P3.2)
// 
//				  [LED Configuration] LED 1  (P2.0)
//				  [LED Configuration] LED 2  (P3.7)
//				  [LED Configuration] LED 3  (P3.6)
//**************************************************************************************

#ifndef	IO_MEM
#define	IO_MEM
#endif
#include "main.h"

/* ---------------------- global variables -------------------------- */


/* ----------------------- main program --------------------------- */
void main(){
    unsigned long a;
    unsigned long b;
    unsigned long c; 
    unsigned char temp;
/* ------------- enable all built-in pull-up resistors ----------------------- */
    P0PU = 0xFF;
    P1PU = 0xFF;
    P2PU = 0xFF;
    P3PU = 0xFF;
    P4PU = 0xFF;
    P5PU = 0xFF;
    
/* ---------------------- system initialization ------------------------ */
    PLLCR0_PLLEN  = 1;                  // enable PLL
    while(Is_PLL_Busy());               // wait for PLL to be ready    
    CLKCR0_SCKPSC = 1;                  // system frequency divided by 2   (48M/2=24MHz)
    CLKCR0_SCKSRC = 1;                  // switch clock source to PLL
    while(Is_SysClk_Busy());            // wait for system clock to be ready
/* ---------------------- pin configuration initialization ------------------------ */
    P2OE_P0 = 1;                        // set P2.0 as output pin
    P2PU_P0 = 0;                        // disable P2.0 built-in pull-up resistor

    P3OE_P7 = 1;                        // set P3.7 as output pin
    P3PU_P7 = 0;                        // disable P3.7 built-in pull-up resistor

    P3OE_P6 = 1;                        // set P3.6 as output pin
    P3PU_P6 = 0;                        // disable P3.6 built-in pull-up resistor

    LED1_out = 1;                       // set LED1 to the light off
    LED2_out = 1;                       // set LED2 to the light off
    LED3_out = 1;                       // set LED3 to the light off

/* ---------------------- DIV initialization ------------------------ */		                                    
    DIV_Init();                         	  // initialize DIV
    //--------------------------------------------------------------------
    while(1){
        if((P3DI & 0x07) != 0x07){
            if(BTN1_in == 0){                   // if press button 1 :  BTN1_in(P3.0)

                a = 0x44B17E22;					// dividend
                b = 0x00000045;                 // divisor
                c = 1;

                *(unsigned long*)&MACC0 = c;    // set quotient c to MACC	
                *(unsigned long*)&MACA0 = a;    // set dividend a to MACA
                *(unsigned long*)&MACB0 = b;    // set divisor b to MACB

                MACCR0_MODE = MAC_Mode_DIV;     // setup MAC to perform division		
		
                MACCR1_START = 1;               // start DIV
                while(MACCR1_START);            // wait for DIV to finish

                c = *(unsigned long*)&MACC0;    // get quotient from MACC
                a = *(unsigned long*)&MACA0;    // get remainder from MACA

                if(Is_MACCR1_Div_Error()){      // check DIV error
                    MACCR1_DIVERR = 1;    	    // clear DIV error flag
                    __ASM("NOP");				// no operation
                }

                if((c != 0x00FEDCBA) || (a != 0x00000000)){     // check DIV results
                    __ASM("NOP");               // no operation
                }
                else{                           // DIV results are correct, light on LED1
                    LED1_out = 0;
                }

            }
            //------------------------------------------------------------------------
            else if(BTN2_in == 0){              // if press button 2 :  BTN2_in(P3.1)

                a = 0x00001234;  				// dividend
                b = 0x00005678;  				// divisor
                c = 0;
                *(unsigned long*)&MACC0 = c;    // set quotient c to MACC
                *(unsigned long*)&MACA0 = a;    // set dividend a to MACA
                *(unsigned long*)&MACB0 = b;    // set divisor b to MACB

                MACCR0_MODE = MAC_Mode_DIV;     // setup MAC to perform division		

                MACCR1_START = 1;               // start DIV
                while(MACCR1_START);            // wait for DIV to finish

                c = *(unsigned long*)&MACC0;    // get quotient from MACC
                a = *(unsigned long*)&MACA0;    // get remainder from MACA

                if(Is_MACCR1_Div_Error()){      // check DIV error
                    MACCR1_DIVERR = 1;    	    // clear DIV error flag
                    __ASM("NOP");               // no operation
                }

                if((c != 0x00000000) || (a != 0x00001234)){     // check DIV results           
                    __ASM("NOP");               // no operation
                }
                else{                           // DIV results are correct, light on LED2
                    LED2_out = 0;
                }
	
            }
            //------------------------------------------------------------------------
            else if(BTN3_in == 0){              // if press button 2 :  BTN3_in(P3.2)

                a = 0xFEDCBA98; 				// dividend
                b = 0x01234567;  				// divisor
                c = 0;

                *(unsigned long*)&MACC0 = c;    // set quotient c to MACC
                *(unsigned long*)&MACA0 = a;    // set dividend a to MACA
                *(unsigned long*)&MACB0 = b;    // set divisor b to MACB

                MACCR0_MODE = MAC_Mode_DIV;     // setup MAC to perform division		

                MACCR1_START = 1;               // start DIV
                while(MACCR1_START);            // wait for DIV to finish

                c = *(unsigned long*)&MACC0;    // get quotient from MACC
                a = *(unsigned long*)&MACA0;    // get remainder from MACA

                if(Is_MACCR1_Div_Error()){      // check DIV error
                    MACCR1_DIVERR = 1;    	    // clear DIV error flag
                    __ASM("NOP");               // no operation
                }

                if((c != 0x000000E0) || (a != 0x00000078)){     // check DIV results
                    __ASM("NOP");               // no operation
                }
                else{                           // DIV results are correct, light on LED3
                    LED3_out = 0;
                }
            }
            //------------------------------------------------------------------------
            while((P3DI & 0x07) != 0x07);       // wait for button released    

            LED1_out = 1;                       // light off all LEDs
            LED2_out = 1;
            LED3_out = 1; 			
        }
    }
}


/* ---------------------- interrupt service routine ------------------------ */
void __interrupt OnlyReti(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n OnlyRetn(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n IntWDT(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n IntSWI(void){        // software reset interrupt service routine
    PRSTR7 = 0x5a;
    PRSTR7 = 0xa5;
    PRSTR7 = 0xc3;
    PRSTR7 = 0x3c;	
}
/* ------------------- end of interrupt service routine ---------------------- */

/* -------------------- SQ7515 interrupt vector ------------------ */
#pragma section const INT_VECTOR1 0xff40
void * const IntTbl1[] = {											
	OnlyReti,				/* 0xff40 : Reserved		*/									
	OnlyReti,				/* 0xff42 : Reserved		*/									
	OnlyReti,				/* 0xff44 : Reserved		*/									
	OnlyReti,				/* 0xff46 : Reserved		*/								
	OnlyReti,				/* 0xff48 : Reserved		*/									
	OnlyReti,				/* 0xff4a : Reserved		*/									
	OnlyReti,				/* 0xff4c : Reserved		*/									
	OnlyReti,				/* 0xff4e : Reserved		*/									
	OnlyReti,				/* 0xff50 : Reserved		*/									
	OnlyReti,				/* 0xff52 : Reserved		*/									
	OnlyReti,				/* 0xff54 : Reserved		*/									
	OnlyReti,				/* 0xff56 : Reserved		*/									
	OnlyReti,				/* 0xff58 : IntTCA7			*/	//interrupt source:TCA7 16-bit timer								
	OnlyReti,				/* 0xff5a : IntTCA6			*/	//interrupt source:TCA6 16-bit timer							
	OnlyReti,				/* 0xff5c : Reserved		*/									
	OnlyReti,				/* 0xff5e : Reserved		*/									
	OnlyReti,				/* 0xff60 : Reserved		*/									
	OnlyReti,				/* 0xff62 : Reserved		*/									
	OnlyReti,				/* 0xff64 : Reserved		*/									
	OnlyReti,				/* 0xff66 : Reserved		*/									
	OnlyReti,				/* 0xff68 : Reserved		*/									
	OnlyReti,				/* 0xff6a : Reserved		*/									
	OnlyReti,				/* 0xff6c : IntTX2			*/	//interrupt source:UART2 TX2								
	OnlyReti,				/* 0xff6e : IntRX2			*/	//interrupt source:UART2 RX2							
	OnlyReti,				/* 0xff70 : Reserved		*/									
	OnlyReti,				/* 0xff72 : Reserved		*/									
	OnlyReti,				/* 0xff74 : Reserved		*/									
	OnlyReti,				/* 0xff76 : IntTCA5			*/	//interrupt source:TCA5 16-bit timer								
	OnlyReti,				/* 0xff78 : IntTCA4			*/	//interrupt source:TCA4 16-bit timer								
	OnlyReti,				/* 0xff7a : Reserved		*/									
	OnlyReti,				/* 0xff7c : Reserved		*/									
	OnlyReti,				/* 0xff7e : Reserved		*/									
	OnlyReti,				/* 0xff80 : Reserved		*/									
	OnlyReti,		    	/* 0xff82 : Reserved		*/									
	OnlyReti,	    		/* 0xff84 : Reserved		*/									
	OnlyReti,				/* 0xff86 : Reserved		*/									
	OnlyReti,				/* 0xff88 : Reserved		*/									
	OnlyReti,				/* 0xff8a : Reserved		*/									
	OnlyReti,				/* 0xff8c : IntSIO1			*/	//interrupt source:SIO 1								
	OnlyReti,				/* 0xff8e : IntI2C1			*/	//interrupt source:I2C1						
	OnlyReti,				/* 0xff90 : IntTX1			*/	//interrupt source:UART1 TX1							
	OnlyReti,				/* 0xff92 : IntRX1			*/	//interrupt source:UART1 RX1								
	OnlyReti,				/* 0xff94 : Reserved		*/									
	OnlyReti,				/* 0xff96 : Reserved		*/									
	OnlyReti,				/* 0xff98 : Reserved		*/									
	OnlyReti,				/* 0xff9a : IntTCA3			*/	//interrupt source:TCA3 16-bit timer							
	OnlyReti,				/* 0xff9c : IntTCA2			*/	//interrupt source:TCA2 16-bit timer								
	OnlyReti,				/* 0xff9e : Reserved		*/									
	OnlyReti,				/* 0xffa0 : IntMAC			*/	//interrupt source:multiplier								
	OnlyReti,				/* 0xffa2 : IntEINT7		*/	//external interrupt7								
	OnlyReti,				/* 0xffa4 : IntEINT6		*/	//external interrupt6								
	OnlyReti,				/* 0xffa6 : IntEINT5		*/	//external interrupt5								
	OnlyReti,				/* 0xffa8 : IntEINT4		*/	//external interrupt4								
	OnlyReti,				/* 0xffaa : Reserved		*/									
	OnlyReti,				/* 0xffac : Reserved		*/									
	OnlyReti,				/* 0xffae : Reserved		*/							
	OnlyReti,				/* 0xffb0 : IntDIC			*/	//interrupt source:DIC								
	OnlyReti,				/* 0xffb2 : Reserved		*/							
	OnlyReti,				/* 0xffb4 : Reserved		*/									
	OnlyReti,				/* 0xffb6 : Reserved		*/									
	OnlyReti,				/* 0xffb8 : Reserved		*/									
	OnlyReti,				/* 0xffba : Reserved		*/									
	OnlyReti,				/* 0xffbc : Reserved		*/									
	OnlyReti,				/* 0xffbe : Reserved		*/									
	OnlyReti,				/* 0xffc0 : Reserved		*/									
	OnlyReti,				/* 0xffc2 : Reserved		*/									
	OnlyReti,				/* 0xffc4 : IntFMC			*/	//interrupt source:flash memory controller								
	OnlyReti,				/* 0xffc6 : Reserved		*/					
	OnlyReti,				/* 0xffc8 : Reserved		*/									
	OnlyReti,				/* 0xffca : Reserved		*/									
	OnlyReti,				/* 0xffcc : Reserved		*/									
	OnlyReti,               /* 0xffce : IntADC			*/	//interrupt source:ADC								
	OnlyReti,				/* 0xffd0 : IntEINT3		*/	//external interrupt3								
	OnlyReti,				/* 0xffd2 : IntEINT2		*/	//external interrupt2								
	OnlyReti,				/* 0xffd4 : IntEINT1		*/	//external interrupt1								
	OnlyReti,				/* 0xffd6 : IntEX0			*/	//external interrupt0								
	OnlyReti,				/* 0xffd8 : IntSIO0			*/	//interrupt source:SIO 0								
	OnlyReti,				/* 0xffda : IntI2C0			*/	//interrupt source:I2C0								
	OnlyReti,				/* 0xffdc : IntTX0			*/	//interrupt source:UART0 TX0								
	OnlyReti,				/* 0xffde : IntRX0			*/	//interrupt source:UART0 RX0								
	OnlyReti,				/* 0xffe0 : Reserved		*/									
	OnlyReti,				/* 0xffe2 : Reserved		*/									
	OnlyReti,				/* 0xffe4 : Reserved		*/									
	OnlyReti,				/* 0xffe6 : IntTCA1			*/	//interrupt source:TCA1 16-bit timer								
	OnlyReti,				/* 0xffe8 : IntTCA0			*/	//interrupt source:TCA0 16-bit timer								
	OnlyReti,				/* 0xffea : Reserved		*/									
	OnlyReti,				/* 0xffec : Reserved		*/									
	OnlyReti,			    /* 0xffee : IntRTC			*/	//interrupt source:real-time clock timer								
	OnlyReti,				/* 0xfff0 : IntTBT			*/  //interrupt source:time base timer				
	OnlyReti,				/* 0xfff2 : Reserved		*/									
	OnlyReti,				/* 0xfff4 : IntCFD			*/	//interrupt source:clock fail detection								
	OnlyReti,				/* 0xfff6 : IntLVD			*/	//interrupt source:low voltage detection							
	IntWDT,					/* 0xfff8 : IntWDT			*/	//non-maskable interrupt source:watchdog timer		
	(void *)0xffff,			/* 0xfffa : Reserved		*/									
	IntSWI,					/* 0xfffc : IntSWI/INTUNDEF */	//non-maskable		
	STARTUP					/* 0xfffe : RESET			*/	//non-maskable			
};
#pragma section const
/* ------------------- end of SQ7515 interrupt vector ---------------------- */

