//SQ7515_I2C0_I2C1 example
//*************************************************************************************
// file			: main.c
// version		: V3.2
// brief		: main program
// description	: this example program is a demostration for I2C1 sending data to I2C0 within a single SQ7615 chip
//			  when press button 1, I2C1 sends command 0x09 and data 0x1234, if I2C0 is received correctly then light on LED1
//      	  when press button 2, I2C1 sends command 0x0A and data 0x55AA, if I2C0 is received correctly then light on LED2
//      	  when press button 3, I2C1 sends command 0x00 and data 0xFFFF, if I2C0 is received correctly then light on LED3
//			  if not yet communicating or receiving error then not light on LEDs
// 
// note			: 1. I2C Master and Slave setting : 
//            	 set I2C0 to slave mode and slave address is 0x68
//			         set P0.5 to SDA0
//		             set P0.6 to SCL0
// 
//			     set I2C1 to master mode
//		             set P2.1 to SDA1
//		             set P2.2 to SCL1
// 
//			  2. SDA0(P0.5) connects to (P2.1)SDA1, 
//				 SCL0(P0.6) connects to (P2.2)SCL1, 
//	             *the above SDA and SCL must be connected with external pull-up resistors
// 
//			  3. [Button Configuration]	button 1 :  BTN1(P3.0), (EINT0)
//							button 2 :  BTN2(P3.1), (EINT1)
//						    button 3 :  BTN3(P3.2), (EINT2)
//
//				  [LED Configuration] LED 1  (P2.0)
//				  [LED Configuration] LED 2  (P3.7)
//				  [LED Configuration] LED 3  (P3.6)
//**************************************************************************************
#ifndef	IO_MEM
#define	IO_MEM
#endif
#include "main.h"

/* ---------------------- global variables -------------------------- */
I2C_MasterPacket i2c_master;
I2C_SlavePacket  i2c_slave;

/* ----------------------- main program --------------------------- */
void main(){
/* ---------------------- system initialization ------------------------ */
    PLLCR0_PLLEN  = 1;                  // enable PLL
    while(PLLCR0_PLLRDY ==0);           // wait for PLL to be ready
    CLKCR0_SCKPSC = 1;                  // system frequency divided by 2   (48M/2=24MHz) 
    CLKCR0_SCKSRC = 1;                  // switch clock source to PLL
    while(CLKCR0_SCKRDY ==0);           // wait for system clock to be ready
/* ---------------------- pin configuration initialization ------------------------ */
    P0PU = 0xFF;                        // enable all built-in pull-up resistors
    P1PU = 0xFF;
    P2PU = 0xFF;
    P3PU = 0xFF;
    P4PU = 0xFF;
    P5PU = 0xFF;

    P2OE_P0 = 1;                        // set P2.0 as output pin
    P2PU_P0 = 0;                        // disable P2.0 built-in pull-up resistor

    P3OE_P7 = 1;                        // set P3.7 as output pin
    P3PU_P7 = 0;                        // disable P3.7 built-in pull-up resistor

    P3OE_P6 = 1;                        // set P3.6 as output pin
    P3PU_P6 = 0;                        // disable P3.6 built-in pull-up resistor

    LED1_out = 1;                       // set LED1 to the light off
    LED2_out = 1;                       // set LED2 to the light off
    LED3_out = 1;                       // set LED3 to the light off
/* ---------------------- EINT initialization ------------------------ */
    EINT_Init();                        // initialize EINT
    EINT_IntEnable();                   // enable EINT interrupt
/* ---------------------- I2C0 initialization ------------------------ */
    I2C0_Init();                        // set I2C0 to slave mode and slave address is 0x68
    I2C0_IntEnable();                   // enable I2C0 interrupt
 /* ---------------------- I2C1 initialization ------------------------ */
    I2C1_Init();                        // set I2C1 to master mode and disable I2C1 interrupt
    i2c_master.adr = SLAVE_ADDR;        // set target slave address to 0x68
    i2c_master.cmd = 0x09;
	//---------------------------------------------------------------------------------
    while(1){								  
        if(Push_BTN1_Flag){      	  		  // if external interrupt flag of button 1 = 1
            Push_BTN1_Flag = 0;				  // clear external interrupt flag 1 in order to wait for the next press
            i2c_master.cmd = 0x09;	          // change command to 0x09
            i2c_master.len = 2;				  // set data length to 2
             
            if(I2C_Master_Read(I2C_1,&i2c_master)==1){
                // I2C read successful
                LED1_out = 0;                 // light on LED1            
            }
            else{
                // I2C read fail
                LED1_out = 1; 				  // light off LED1
            }
        }
		//----------------------------------------------------------------------------
        if(Push_BTN2_Flag){      	  		  // if external interrupt flag of button 2 = 1
            Push_BTN2_Flag = 0;				  // clear external interrupt flag 2 in order to wait for the next press
            i2c_master.cmd = 0x0a;      	  // change command to 0x0A
            i2c_master.len = 2;         	  // set data length to 2
            if(I2C_Master_Read(I2C_1,&i2c_master)==1){
                // I2C read successful
                LED2_out = 0;				  // LED2 light on  
            }
            else{
                // I2C read fail
                LED2_out = 1;				  // light off LED2
            }
        }
		//----------------------------------------------------------------------------
        if(Push_BTN3_Flag){      	  		  // if external interrupt flag of button 3 = 1
            Push_BTN3_Flag = 0;				  // clear external interrupt flag 3 in order to wait for the next press
            i2c_master.cmd = 0x00;      	  // change command to 0x00
            i2c_master.len = 2;         	  // set data length to 2
            if(I2C_Master_Read(I2C_1,&i2c_master)==1){
                // I2C read successful
                LED3_out = 0;				  // LED3 light on 
            }
            else{
                // I2C read fail
                LED3_out = 1;				  // light off LED3
            }
        }		
        //------------------------------------------------------------------------------
    }
}


/* ---------------------- interrupt service routine ------------------------ */
void __interrupt OnlyReti(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n OnlyRetn(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n IntWDT(void){
    __asm("NOP");                       // no operation
}

void __interrupt_n IntSWI(void){        // software reset interrupt service routine
    PRSTR7 = 0x5a;
    PRSTR7 = 0xa5;
    PRSTR7 = 0xc3;
    PRSTR7 = 0x3c;	
}
/* ------------------- end of interrupt service routine ---------------------- */


/* -------------------- SQ7515 interrupt vector ------------------ */
#pragma section const INT_VECTOR1 0xff40
void * const IntTbl1[] = {											
	OnlyReti,				/* 0xff40 : Reserved		*/									
	OnlyReti,				/* 0xff42 : Reserved		*/									
	OnlyReti,				/* 0xff44 : Reserved		*/									
	OnlyReti,				/* 0xff46 : Reserved		*/								
	OnlyReti,				/* 0xff48 : Reserved		*/									
	OnlyReti,				/* 0xff4a : Reserved		*/									
	OnlyReti,				/* 0xff4c : Reserved		*/									
	OnlyReti,				/* 0xff4e : Reserved		*/									
	OnlyReti,				/* 0xff50 : Reserved		*/									
	OnlyReti,				/* 0xff52 : Reserved		*/									
	OnlyReti,				/* 0xff54 : Reserved		*/									
	OnlyReti,				/* 0xff56 : Reserved		*/									
	OnlyReti,				/* 0xff58 : IntTCA7			*/	//interrupt source:TCA7 16-bit timer								
	OnlyReti,				/* 0xff5a : IntTCA6			*/	//interrupt source:TCA6 16-bit timer							
	OnlyReti,				/* 0xff5c : Reserved		*/									
	OnlyReti,				/* 0xff5e : Reserved		*/									
	OnlyReti,				/* 0xff60 : Reserved		*/									
	OnlyReti,				/* 0xff62 : Reserved		*/									
	OnlyReti,				/* 0xff64 : Reserved		*/									
	OnlyReti,				/* 0xff66 : Reserved		*/									
	OnlyReti,				/* 0xff68 : Reserved		*/									
	OnlyReti,				/* 0xff6a : Reserved		*/									
	OnlyReti,				/* 0xff6c : IntTX2			*/	//interrupt source:UART2 TX2								
	OnlyReti,				/* 0xff6e : IntRX2			*/	//interrupt source:UART2 RX2							
	OnlyReti,				/* 0xff70 : Reserved		*/									
	OnlyReti,				/* 0xff72 : Reserved		*/									
	OnlyReti,				/* 0xff74 : Reserved		*/									
	OnlyReti,				/* 0xff76 : IntTCA5			*/	//interrupt source:TCA5 16-bit timer								
	OnlyReti,				/* 0xff78 : IntTCA4			*/	//interrupt source:TCA4 16-bit timer								
	OnlyReti,				/* 0xff7a : Reserved		*/									
	OnlyReti,				/* 0xff7c : Reserved		*/									
	OnlyReti,				/* 0xff7e : Reserved		*/									
	OnlyReti,				/* 0xff80 : Reserved		*/									
	OnlyReti,		    	/* 0xff82 : Reserved		*/									
	OnlyReti,	    		/* 0xff84 : Reserved		*/									
	OnlyReti,				/* 0xff86 : Reserved		*/									
	OnlyReti,				/* 0xff88 : Reserved		*/									
	OnlyReti,				/* 0xff8a : Reserved		*/									
	OnlyReti,				/* 0xff8c : IntSIO1			*/	//interrupt source:SIO 1								
	IntI2C1,				/* 0xff8e : IntI2C1			*/	//interrupt source:I2C1						
	OnlyReti,				/* 0xff90 : IntTX1			*/	//interrupt source:UART1 TX1							
	OnlyReti,				/* 0xff92 : IntRX1			*/	//interrupt source:UART1 RX1								
	OnlyReti,				/* 0xff94 : Reserved		*/									
	OnlyReti,				/* 0xff96 : Reserved		*/									
	OnlyReti,				/* 0xff98 : Reserved		*/									
	OnlyReti,				/* 0xff9a : IntTCA3			*/	//interrupt source:TCA3 16-bit timer							
	OnlyReti,				/* 0xff9c : IntTCA2			*/	//interrupt source:TCA2 16-bit timer								
	OnlyReti,				/* 0xff9e : Reserved		*/									
	OnlyReti,				/* 0xffa0 : IntMAC			*/	//interrupt source:multiplier								
	OnlyReti,				/* 0xffa2 : IntEXT7			*/	//external interrupt7								
	OnlyReti,				/* 0xffa4 : IntEXT6			*/	//external interrupt6								
	OnlyReti,				/* 0xffa6 : IntEXT5			*/	//external interrupt5								
	OnlyReti,				/* 0xffa8 : IntEXT4			*/	//external interrupt4								
	OnlyReti,				/* 0xffaa : Reserved		*/									
	OnlyReti,				/* 0xffac : Reserved		*/									
	OnlyReti,				/* 0xffae : Reserved		*/							
	OnlyReti,				/* 0xffb0 : IntDIC			*/	//interrupt source:DIC								
	OnlyReti,				/* 0xffb2 : Reserved		*/							
	OnlyReti,				/* 0xffb4 : Reserved		*/									
	OnlyReti,				/* 0xffb6 : Reserved		*/									
	OnlyReti,				/* 0xffb8 : Reserved		*/									
	OnlyReti,				/* 0xffba : Reserved		*/									
	OnlyReti,				/* 0xffbc : Reserved		*/									
	OnlyReti,				/* 0xffbe : Reserved		*/									
	OnlyReti,				/* 0xffc0 : Reserved		*/									
	OnlyReti,				/* 0xffc2 : Reserved		*/									
	OnlyReti,				/* 0xffc4 : IntFMC			*/	//interrupt source:flash memory controller								
	OnlyReti,				/* 0xffc6 : Reserved		*/					
	OnlyReti,				/* 0xffc8 : Reserved		*/									
	OnlyReti,				/* 0xffca : Reserved		*/									
	OnlyReti,				/* 0xffcc : Reserved		*/									
	OnlyReti,               /* 0xffce : IntADC			*/	//interrupt source:ADC								
	OnlyReti,				/* 0xffd0 : IntEXT3			*/	//external interrupt3								
	IntEXT2,				/* 0xffd2 : IntEXT2			*/	//external interrupt2								
	IntEXT1,				/* 0xffd4 : IntEXT1			*/	//external interrupt1								
	IntEXT0,				/* 0xffd6 : IntEXT0			*/	//external interrupt0								
	OnlyReti,				/* 0xffd8 : IntSIO0			*/	//interrupt source:SIO 0								
	IntI2C0,				/* 0xffda : IntI2C0			*/	//interrupt source:I2C0								
	OnlyReti,				/* 0xffdc : IntTX0			*/	//interrupt source:UART0 TX0								
	OnlyReti,				/* 0xffde : IntRX0			*/	//interrupt source:UART0 RX0								
	OnlyReti,				/* 0xffe0 : Reserved		*/									
	OnlyReti,				/* 0xffe2 : Reserved		*/									
	OnlyReti,				/* 0xffe4 : Reserved		*/									
	OnlyReti,				/* 0xffe6 : IntTCA1			*/	//interrupt source:TCA1 16-bit timer								
	OnlyReti,				/* 0xffe8 : IntTCA0			*/	//interrupt source:TCA0 16-bit timer								
	OnlyReti,				/* 0xffea : Reserved		*/									
	OnlyReti,				/* 0xffec : Reserved		*/									
	OnlyReti,			    /* 0xffee : IntRTC			*/	//interrupt source:real-time clock timer								
	OnlyReti,				/* 0xfff0 : IntTBT			*/  //interrupt source:time base timer				
	OnlyReti,				/* 0xfff2 : Reserved		*/									
	OnlyReti,				/* 0xfff4 : IntCFD			*/	//interrupt source:clock fail detection								
	OnlyReti,				/* 0xfff6 : IntLVD			*/	//interrupt source:low voltage detection							
	IntWDT,					/* 0xfff8 : IntWDT			*/	//non-maskable interrupt source:watchdog timer		
	(void *)0xffff,			/* 0xfffa : Reserved		*/									
	IntSWI,					/* 0xfffc : IntSWI/INTUNDEF */	//non-maskable		
	STARTUP					/* 0xfffe : RESET			*/	//non-maskable			
};
#pragma section const
/* ------------------- end of SQ7515 interrupt vector ---------------------- */

