//SQ7615_OPMod example
//-Sleep Mode_RTC_wake up
//-Deep Sleep_RTC_wake up
//*************************************************************************************
// file			: main.c
// version		: V3.1
// brief		: main program
// description	: SQ7615 has three kinds of operation mode :
//			  (1) Normal mode
//			  (2) Sleep mode
//			  (3) Deep sleep mode
//
//			  this example starts with Normal mode and LED1 is on 
//      	  when press button 2, light off LED1 first, then enter Sleep mode and light on LED1 again when return to Normal mode by RTC interrupt 
//      	  when press button 3, light off LED1 first, then enter Deep Sleep mode and light on LED1 again when return to Normal mode by RTC interrupt 
// 
// note			: 	  ***[external low frequency crystal Configuration] connect external 32.768KHz crystal with P4.0 and P4.1
//			      [Button Configuration]	
//						    button 2 :  BTN2_in(P3.1) [IntEXT1]
//						    button 3 :  BTN3_in(P3.2) [IntEXT2]
//				  [LED Configuration] LED 1  (P2.0)
//**************************************************************************************
#ifndef	IO_MEM
#define	IO_MEM
#endif
#include "main.h"

RTC_Perpetual_Calendar_Time  rtc;                             
RTC_ALARM  rtc_alarm;

/* ---------------------- main program ------------------------ */
void main()
{
/* ---------------------- system initialization ------------------------ */
	PLLCR0_PLLEN  = 1;                     	  // enable PLL
	while(PLLCR0_PLLRDY ==0);				  // wait for PLL to be ready
	CLKCR0_SCKPSC = 1;                        // system frequency divided by 2   (48M/2=24MHz) 
 	CLKCR0_SCKSRC = 1;                     	  // switch clock source to PLL	
	while(CLKCR0_SCKRDY ==0);				  // wait for system clock to be ready

	P4FC1   = 0x03;                        	  // set P4.0 to LXOUT and set P4.1 to LXIN 
	P4FC2   = 0x03;
    CLKCR1_LXEN = 1;                     	  // enable external low frequency crystal  
    while(CLKCR1_LXRDY == 0);            	  // wait for external low frequency crystal ready
    CLKCR0_SCKLF = 1;                    	  // set external low frequency crystal as system clock  
/* ---------------------- pin configuration initialization ------------------------ */
    P2OE  = 0x07;                        	  // set P2.0, P2.1 and P2.2 as output pins
    P3OE  = 0x00;                        	  // set P3.0 as input pin
	P3PU |= 0x07;                       	  // enable P3.0 built-in pull-up resistor
	LED1_out = 0;							  // light on LED1
	LED2_out = 1;							  // set LED2 to the light off
	LED3_out = 1;							  // set LED3 to the light off
/* ---------------------- EINT initialization ------------------------ */		
	EINT_Init();                        	  // EINT initialization
	EINT_IntEnable();                   	  // enable EINT interrupt
/* ---------------------- RTC initialization ------------------------ */		
    RTC_Init();                         	  // RTC initialization
//------------------------------------------------------------------------------	
    while(1){
	
		if(Push_BTN2_Flag){      	  		  // if external interrupt flag of button 2 = 1
		    Push_BTN2_Flag = 0;	  			  // clear external interrupt flag of button 2
 			LED1_out = 1;               	  // light off LED1
		/* ------------------------- RTC setting ------------------------- */	
			// set current time : 2019/11/18 Monday 15:06:50
			RTC_Setup();                	  // set current time
			
			// set alarm time : 18 Monday 15:08:00
			Alarm_Setup();					  // set alarm time
	        RTC_IntEnable(RTC_AIE,1);		  // enable RTC interrupt
			RTC_Start();					  // start RTC
			
		/* ---------------------- enter Sleep mode ------------------------ */		
			PMR_DSM = 0;                	  // enter Sleep mode with SLEEP instruction    			
	        __ASM("SLEEP");             	  // SLEEP instruction
			
		/* ---------------------- return to Normal mode ------------------------ */
            __ASM("NOP");               	  // **add no operation after system wake-up, please do not set the program breakpoint in this line in debug mode
	        
			RTC_Refresh(&rtc); 			  	  // update RTC time
			RTC_IntDisable();				  // disable RTC interrupt
			RTC_Stop();                 	  // stop RTC
		
			LED1_out = 0;               	  // light on LED1(P2.0)	
		}
		//--------------------------------------------------------------------------------------------------------
		else if(Push_BTN3_Flag){			  // if external interrupt flag of button 3 = 1
			Push_BTN3_Flag = 0;	  			  // clear external interrupt flag of button 3
 			LED1_out = 1;               	  // light off LED1
		/* ------------------------- RTC setting ------------------------- */	
			RTC_Setup();                	  // set current time
			Alarm_Setup();					  // set alarm time
			RTC_IntEnable(RTC_AIE,1);		  // enable RTC interrupt
			RTC_Start();					  // start RTC
			
			CLKCR1_HIRCEN = 1;
	    /* ---------------------- Deep Sleep mode initialization --------------------- */   
			PONCR_BROREN = 0; 				  // disable BROR
			
		/* ---------------------- enter Deep Sleep mode ------------------------ */	
			PMR_DSM = 1;                	  // enter Deep Sleep mode with SLEEP instruction    			
	        __ASM("SLEEP");             	  // SLEEP instruction
			
		/* ---------------------- return to Normal mode ------------------------ */
            __ASM("NOP");               	  // **add no operation after system wake-up, please do not set the program breakpoint in this line in debug mode
			RTC_Refresh(&rtc); 			  	  // update RTC time
			RTC_IntDisable();				  // disable RTC interrupt
	        RTC_Stop();                 	  // stop RTC

			LED1_out = 0;               	  // light on LED1(P2.0)

		}
		
    }
}


/* ---------------------- interrupt service routine ------------------------ */
void __interrupt OnlyReti(void)
{
	__asm("NOP"); 							  // no operation
}

void __interrupt_n OnlyRetn(void)
{
	__asm("NOP");							  // no operation
}

void __interrupt_n IntWDT(void)
{
	__asm("NOP");							  // no operation
}

void __interrupt_n IntSWI(void)               // software reset interrupt service routine
{
	PRSTR7 = 0x5a;
	PRSTR7 = 0xa5;
	PRSTR7 = 0xc3;
	PRSTR7 = 0x3c;	
}
/* ------------------- end of interrupt service routine ---------------------- */



/* -------------------- SQ7615 interrupt vector ------------------ */
#pragma section const INT_VECTOR1 0xff40
void * const IntTbl1[] = {											
	OnlyReti,				/* 0xff40 : Reserved		*/									
	OnlyReti,				/* 0xff42 : Reserved		*/									
	OnlyReti,				/* 0xff44 : Reserved		*/									
	OnlyReti,				/* 0xff46 : Reserved		*/								
	OnlyReti,				/* 0xff48 : Reserved		*/									
	OnlyReti,				/* 0xff4a : Reserved		*/									
	OnlyReti,				/* 0xff4c : Reserved		*/									
	OnlyReti,				/* 0xff4e : Reserved		*/									
	OnlyReti,				/* 0xff50 : Reserved		*/									
	OnlyReti,				/* 0xff52 : Reserved		*/									
	OnlyReti,				/* 0xff54 : Reserved		*/									
	OnlyReti,				/* 0xff56 : Reserved		*/									
	OnlyReti,				/* 0xff58 : IntTCA7			*/	//interrupt source:TCA7 16-bit timer								
	OnlyReti,				/* 0xff5a : IntTCA6			*/	//interrupt source:TCA6 16-bit timer							
	OnlyReti,				/* 0xff5c : Reserved		*/									
	OnlyReti,				/* 0xff5e : Reserved		*/									
	OnlyReti,				/* 0xff60 : Reserved		*/									
	OnlyReti,				/* 0xff62 : Reserved		*/									
	OnlyReti,				/* 0xff64 : Reserved		*/									
	OnlyReti,				/* 0xff66 : Reserved		*/									
	OnlyReti,				/* 0xff68 : Reserved		*/									
	OnlyReti,				/* 0xff6a : Reserved		*/									
	OnlyReti,				/* 0xff6c : IntTX2			*/	//interrupt source:UART2 TX2								
	OnlyReti,				/* 0xff6e : IntRX2			*/	//interrupt source:UART2 RX2							
	OnlyReti,				/* 0xff70 : Reserved		*/									
	OnlyReti,				/* 0xff72 : Reserved		*/									
	OnlyReti,				/* 0xff74 : Reserved		*/									
	OnlyReti,				/* 0xff76 : IntTCA5			*/	//interrupt source:TCA5 16-bit timer								
	OnlyReti,				/* 0xff78 : IntTCA4			*/	//interrupt source:TCA4 16-bit timer								
	OnlyReti,				/* 0xff7a : Reserved		*/									
	OnlyReti,				/* 0xff7c : Reserved		*/									
	OnlyReti,				/* 0xff7e : Reserved		*/									
	OnlyReti,				/* 0xff80 : Reserved		*/									
	OnlyReti,		    	/* 0xff82 : Reserved		*/									
	OnlyReti,	    		/* 0xff84 : Reserved		*/									
	OnlyReti,				/* 0xff86 : Reserved		*/									
	OnlyReti,				/* 0xff88 : Reserved		*/									
	OnlyReti,				/* 0xff8a : Reserved		*/									
	OnlyReti,				/* 0xff8c : IntSIO1			*/	//interrupt source:SIO 1								
	OnlyReti,				/* 0xff8e : IntI2C1			*/	//interrupt source:I2C1						
	OnlyReti,				/* 0xff90 : IntTX1			*/	//interrupt source:UART1 TX1							
	OnlyReti,				/* 0xff92 : IntRX1			*/	//interrupt source:UART1 RX1								
	OnlyReti,				/* 0xff94 : Reserved		*/									
	OnlyReti,				/* 0xff96 : Reserved		*/									
	OnlyReti,				/* 0xff98 : Reserved		*/									
	OnlyReti,				/* 0xff9a : IntTCA3			*/	//interrupt source:TCA3 16-bit timer							
	OnlyReti,				/* 0xff9c : IntTCA2			*/	//interrupt source:TCA2 16-bit timer								
	OnlyReti,				/* 0xff9e : Reserved		*/									
	OnlyReti,				/* 0xffa0 : IntMAC			*/	//interrupt source:multiplier								
	OnlyReti,				/* 0xffa2 : IntEXT7			*/	//external interrupt7								
	OnlyReti,				/* 0xffa4 : IntEXT6			*/	//external interrupt6								
	OnlyReti,				/* 0xffa6 : IntEXT5			*/	//external interrupt5								
	OnlyReti,				/* 0xffa8 : IntEXT4			*/	//external interrupt4								
	OnlyReti,				/* 0xffaa : Reserved		*/									
	OnlyReti,				/* 0xffac : Reserved		*/									
	OnlyReti,				/* 0xffae : Reserved		*/							
	OnlyReti,				/* 0xffb0 : IntDIC			*/	//interrupt source:DIC								
	OnlyReti,				/* 0xffb2 : Reserved		*/							
	OnlyReti,				/* 0xffb4 : Reserved		*/									
	OnlyReti,				/* 0xffb6 : Reserved		*/									
	OnlyReti,				/* 0xffb8 : Reserved		*/									
	OnlyReti,				/* 0xffba : Reserved		*/									
	OnlyReti,				/* 0xffbc : Reserved		*/									
	OnlyReti,				/* 0xffbe : Reserved		*/									
	OnlyReti,				/* 0xffc0 : Reserved		*/									
	OnlyReti,				/* 0xffc2 : Reserved		*/									
	OnlyReti,				/* 0xffc4 : IntFMC			*/	//interrupt source:flash memory controller								
	OnlyReti,				/* 0xffc6 : Reserved		*/					
	OnlyReti,				/* 0xffc8 : Reserved		*/									
	OnlyReti,				/* 0xffca : Reserved		*/									
	OnlyReti,				/* 0xffcc : Reserved		*/									
	OnlyReti,               /* 0xffce : IntADC			*/	//interrupt source:ADC								
	OnlyReti,				/* 0xffd0 : IntEXT3			*/	//external interrupt3								
	IntEXT2,				/* 0xffd2 : IntEXT2			*/	//external interrupt2								
	IntEXT1,				/* 0xffd4 : IntEXT1			*/	//external interrupt1								
	IntEXT0,				/* 0xffd6 : IntEXT0			*/	//external interrupt0								
	OnlyReti,				/* 0xffd8 : IntSIO0			*/	//interrupt source:SIO 0								
	OnlyReti,				/* 0xffda : IntI2C0			*/	//interrupt source:I2C0								
	OnlyReti,				/* 0xffdc : IntTX0			*/	//interrupt source:UART0 TX0								
	OnlyReti,				/* 0xffde : IntRX0			*/	//interrupt source:UART0 RX0								
	OnlyReti,				/* 0xffe0 : Reserved		*/									
	OnlyReti,				/* 0xffe2 : Reserved		*/									
	OnlyReti,				/* 0xffe4 : Reserved		*/									
	OnlyReti,				/* 0xffe6 : IntTCA1			*/	//interrupt source:TCA1 16-bit timer								
	OnlyReti,				/* 0xffe8 : IntTCA0			*/	//interrupt source:TCA0 16-bit timer								
	OnlyReti,				/* 0xffea : Reserved		*/									
	OnlyReti,				/* 0xffec : Reserved		*/									
	IntRTC,			    	/* 0xffee : IntRTC			*/	//interrupt source:real-time clock timer								
	OnlyReti,				/* 0xfff0 : IntTBT			*/  //interrupt source:time base timer				
	OnlyReti,				/* 0xfff2 : Reserved		*/									
	OnlyReti,				/* 0xfff4 : IntCFD			*/	//interrupt source:clock fail detection								
	OnlyReti,				/* 0xfff6 : IntLVD			*/	//interrupt source:low voltage detection							
	IntWDT,					/* 0xfff8 : IntWDT			*/	//non-maskable interrupt source:watchdog timer		
	(void *)0xffff,			/* 0xfffa : Reserved		*/									
	IntSWI,					/* 0xfffc : IntSWI/INTUNDEF */	//non-maskable		
	STARTUP					/* 0xfffe : RESET			*/	//non-maskable			
};
#pragma section const
/* ------------------- end of SQ7615 interrupt vector ---------------------- */