//SQ7617_SIO example
//*************************************************************************************
// file			: main.c
// version		: V1.0
// brief		: main program
// description	: this example program is a demostration for SIO1 sending data to SIO0 within a single SQ7617 chip
//			  when press button 1, SIO1 sends data 0x55, if SIO0 is received correctly then light on LED1
//      	  when press button 2, SIO1 sends data 0xAA, if SIO0 is received correctly then light on LED2
//      	  when press button 3, SIO1 sends data 0xF0, if SIO0 is received correctly then light on LED3
// 
// note			: 1. set SIO1 as Master 
//			         set P0.2 to SCLK1
//		             set P0.0 to SI1
//					 set P0.1 to SO1
// 
//			  2. set SIO0 as Slave 
//			         set P0.6 to SCLK0
//		             set P0.5 to SI0
//					 set P0.4 to SO0
// 
//			  3. SCLK1(P0.2) connects to SCLK0(P0.6)
//				 SI1(P0.0) connects to SI0(P0.4)
//				 SO1(P0.1) connects to SO0(P0.5)
// 				 set P4.3 to simulate SS pin of SPI interface
// 
//			  4. [Button Configuration]	button 1 :  BTN1_in(P3.0)  [IntEXT0]
//						    button 2 :  BTN2_in(P3.1)  [IntEXT1]
//						    button 3 :  BTN3_in(P3.7)  [IntEXT7]
// 
//				  [LED Configuration] LED 1  (P2.0)
//				  [LED Configuration] LED 2  (P2.1)
//				  [LED Configuration] LED 3  (P2.2)
//**************************************************************************************
#ifndef	IO_MEM
#define	IO_MEM
#endif
#include "main.h"

/* ---------------------- global variables -------------------------- */

unsigned char m_rx;

/* ----------------------- main program --------------------------- */
void main()
{
/* ---------------------- system initialization ------------------------ */
	PLLCR0_PLLEN  = 1;                     	  // enable PLL
	while(PLLCR0_PLLRDY ==0);				  // wait for PLL to be ready
	CLKCR0_SCKPSC = 1;                        // system frequency divided by 2   (48M/2=24MHz) 
    CLKCR0_SCKSRC = 1;                        // switch clock source to PLL
	while(CLKCR0_SCKRDY ==0);				  // wait for system clock to be ready
/* ---------------------- pin configuration initialization ------------------------ */
    P2OE  = 0x07;                        	  // set P2.0, P2.1 and P2.2 as output pins
    P3OE  = 0x7C;                        	  // set P3.0, P3.1 and P3.7 as input pins
	P3PU |= 0x83;                       	  // enable P3.0, P3.1 and P3.7 built-in pull-up resistors
	P4OE_P3 = 1;							  // set P4.3 as output pin
	P4DO_P3 = 1;							  // set P4.3 to output high
	LED1_out = 1;							  // set LED1 to the light off
	LED2_out = 1;							  // set LED2 to the light off
	LED3_out = 1;							  // set LED3 to the light off
/* ---------------------- EINT initialization ------------------------ */
	EINT_Init();                        	  // initialize EINT
	EINT_IntEnable();                   	  // enable external interrupt
/* ---------------------- SIO1 initialization ------------------------ */
    SIO1_Init();                        	  // initialize SIO1 to SIO Master
/* ---------------------- SIO0 initialization ------------------------ */
	SIO0_Init();                        	  // initialize SIO0 to SIO Slave 
	SIO_IntEnable(SIO_0);               	  // enable SIO0 interrupt
/* ---------------------- start SIO0 --------------------------- */	
	SIO_Start(SIO_0);                   	  // start SIO0 to receive data
//---------------------------------------------------------------------
    while(1){
		if(Push_BTN1_Flag){      	  		  // external interrupt flag of button 1BTN1(IntEXT0)=1
			SIO_Start(SIO_0);                 // start SIO0 to receive data
			Push_BTN1_Flag = 0;				  // clear external interrupt flag 1 in order to wait for the next press
			SIO_ByteSend(SIO_1,0x55);	  	  // SIO1 transmits data 0x55
			m_rx = SIO_ByteRead(SIO_1);	      // SIO1 receives data

			if(m_rx == 0x76)          		  // if SIO1 receives data 0x76
			{
				LED1_out = 0;           	  // light on LED1
			}
			else
			{
				LED1_out = 1;           	  // light off LED1
			}
		SIO_Stop(SIO_0);                   	  // stop SIO0	
		}
		//---------------------------------------------------------------
		if(Push_BTN2_Flag){      	  		  // external interrupt flag of button 2BTN2(IntEXT1)=1
			SIO_Start(SIO_0);                 // start SIO0 to receive data
			Push_BTN2_Flag = 0;				  // clear external interrupt flag 2 in order to wait for the next press
			SIO_ByteSend(SIO_1, 0xAA);		  // SIO1 transmits data 0xAA
			m_rx = SIO_ByteRead(SIO_1);		  // SIO1 receives data
			
			if(m_rx == 0x17)          		  // Arx dataΪ0x17
			{
				LED2_out = 0;           	  // light on LED2
			}
			else
			{
				LED2_out = 1;                 // light off LED2
			}
		SIO_Stop(SIO_0);                   	  // stop SIO0	
		}
		//---------------------------------------------------------------
		if(Push_BTN3_Flag){      	  		  // external interrupt flag of button 3BTN3(IntEXT7)=1
			SIO_Start(SIO_0);                 // start SIO0 to receive data
			Push_BTN3_Flag = 0;				  // clear external interrupt flag 3 in order to wait for the next press
			SIO_ByteSend(SIO_1,0xF0);		  // SIO1 transmits data 0xF0
			m_rx = SIO_ByteRead(SIO_1);		  // SIO1 receives data
			
			if(m_rx == 0x98)          		  // if SIO1 receives data 0x98
			{
				LED3_out = 0;           	  // light on LED3
			}
			else
			{				
				LED3_out = 1;           	  // light off LED3
			}
		SIO_Stop(SIO_0);                   	  // stop SIO0	
		}
	
    }
}


/* ---------------------- interrupt service routine ------------------------ */
void __interrupt OnlyReti(void)
{
	__asm("NOP"); 							  // no operation
}

void __interrupt_n OnlyRetn(void)
{
	__asm("NOP");							  // no operation
}

void __interrupt_n IntWDT(void)
{
	__asm("NOP");							  // no operation
}

void __interrupt_n IntSWI(void)			  // software reset interrupt service routine
{
	PRSTR7 = 0x5a;
	PRSTR7 = 0xa5;
	PRSTR7 = 0xc3;
	PRSTR7 = 0x3c;	
}
/* ------------------- end of interrupt service routine ---------------------- */

/* -------------------- SQ7617 interrupt vector ------------------ */
#pragma section const INT_VECTOR1 0xff40
void * const IntTbl1[] = {											
	OnlyReti,				/* 0xff40 : Reserved		*/									
	OnlyReti,				/* 0xff42 : Reserved		*/									
	OnlyReti,				/* 0xff44 : Reserved		*/									
	OnlyReti,				/* 0xff46 : Reserved		*/								
	OnlyReti,				/* 0xff48 : Reserved		*/									
	OnlyReti,				/* 0xff4a : Reserved		*/									
	OnlyReti,				/* 0xff4c : Reserved		*/									
	OnlyReti,				/* 0xff4e : Reserved		*/									
	OnlyReti,				/* 0xff50 : Reserved		*/									
	OnlyReti,				/* 0xff52 : Reserved		*/									
	OnlyReti,				/* 0xff54 : Reserved		*/									
	OnlyReti,				/* 0xff56 : Reserved		*/									
	OnlyReti,				/* 0xff58 : IntTCA7			*/	//interrupt source:TCA7 16-bit timer								
	OnlyReti,				/* 0xff5a : IntTCA6			*/	//interrupt source:TCA6 16-bit timer							
	OnlyReti,				/* 0xff5c : Reserved		*/									
	OnlyReti,				/* 0xff5e : Reserved		*/									
	OnlyReti,				/* 0xff60 : Reserved		*/									
	OnlyReti,				/* 0xff62 : Reserved		*/									
	OnlyReti,				/* 0xff64 : Reserved		*/									
	OnlyReti,				/* 0xff66 : Reserved		*/									
	OnlyReti,				/* 0xff68 : Reserved		*/									
	OnlyReti,				/* 0xff6a : Reserved		*/									
	OnlyReti,				/* 0xff6c : IntTX2			*/	//interrupt source:UART2 TX2								
	OnlyReti,				/* 0xff6e : IntRX2			*/	//interrupt source:UART2 RX2							
	OnlyReti,				/* 0xff70 : Reserved		*/									
	OnlyReti,				/* 0xff72 : Reserved		*/									
	OnlyReti,				/* 0xff74 : Reserved		*/									
	OnlyReti,				/* 0xff76 : IntTCA5			*/	//interrupt source:TCA5 16-bit timer								
	OnlyReti,				/* 0xff78 : IntTCA4			*/	//interrupt source:TCA4 16-bit timer								
	OnlyReti,				/* 0xff7a : Reserved		*/									
	OnlyReti,				/* 0xff7c : Reserved		*/									
	OnlyReti,				/* 0xff7e : Reserved		*/									
	OnlyReti,				/* 0xff80 : Reserved		*/									
	OnlyReti,		    	/* 0xff82 : Reserved		*/									
	OnlyReti,	    		/* 0xff84 : Reserved		*/									
	OnlyReti,				/* 0xff86 : Reserved		*/									
	OnlyReti,				/* 0xff88 : Reserved		*/									
	OnlyReti,				/* 0xff8a : Reserved		*/									
	IntSIO1,				/* 0xff8c : IntSIO1			*/	//interrupt source:SIO 1								
	OnlyReti,				/* 0xff8e : IntI2C1			*/	//interrupt source:I2C1						
	OnlyReti,				/* 0xff90 : IntTX1			*/	//interrupt source:UART1 TX1							
	OnlyReti,				/* 0xff92 : IntRX1			*/	//interrupt source:UART1 RX1								
	OnlyReti,				/* 0xff94 : Reserved		*/									
	OnlyReti,				/* 0xff96 : Reserved		*/									
	OnlyReti,				/* 0xff98 : Reserved		*/									
	OnlyReti,				/* 0xff9a : IntTCA3			*/	//interrupt source:TCA3 16-bit timer							
	OnlyReti,				/* 0xff9c : IntTCA2			*/	//interrupt source:TCA2 16-bit timer								
	OnlyReti,				/* 0xff9e : Reserved		*/									
	OnlyReti,				/* 0xffa0 : IntMAC			*/	//interrupt source:multiplier								
	IntEXT7,				/* 0xffa2 : IntEXT7			*/	//external interrupt7								
	OnlyReti,				/* 0xffa4 : IntEXT6			*/	//external interrupt6								
	OnlyReti,				/* 0xffa6 : IntEXT5			*/	//external interrupt5								
	OnlyReti,				/* 0xffa8 : IntEXT4			*/	//external interrupt4								
	OnlyReti,				/* 0xffaa : Reserved		*/									
	OnlyReti,				/* 0xffac : Reserved		*/									
	OnlyReti,				/* 0xffae : Reserved		*/							
	OnlyReti,				/* 0xffb0 : IntDIC			*/	//interrupt source:DIC								
	OnlyReti,				/* 0xffb2 : Reserved		*/							
	OnlyReti,				/* 0xffb4 : Reserved		*/									
	OnlyReti,				/* 0xffb6 : Reserved		*/									
	OnlyReti,				/* 0xffb8 : Reserved		*/									
	OnlyReti,				/* 0xffba : Reserved		*/									
	OnlyReti,				/* 0xffbc : Reserved		*/									
	OnlyReti,				/* 0xffbe : Reserved		*/									
	OnlyReti,				/* 0xffc0 : Reserved		*/									
	OnlyReti,				/* 0xffc2 : Reserved		*/									
	OnlyReti,				/* 0xffc4 : IntFMC			*/	//interrupt source:flash memory controller								
	OnlyReti,				/* 0xffc6 : Reserved		*/					
	OnlyReti,				/* 0xffc8 : Reserved		*/									
	OnlyReti,				/* 0xffca : Reserved		*/									
	OnlyReti,				/* 0xffcc : Reserved		*/									
	OnlyReti,               /* 0xffce : IntADC			*/	//interrupt source:ADC								
	OnlyReti,				/* 0xffd0 : IntEXT3			*/	//external interrupt3								
	OnlyReti,				/* 0xffd2 : IntEXT2			*/	//external interrupt2								
	IntEXT1,				/* 0xffd4 : IntEXT1			*/	//external interrupt1								
	IntEXT0,				/* 0xffd6 : IntEXT0			*/	//external interrupt0								
	IntSIO0,				/* 0xffd8 : IntSIO0			*/	//interrupt source:SIO 0								
	OnlyReti,				/* 0xffda : IntI2C0			*/	//***I2C0 interrupt reserved for internal EEPROM use only***									
	OnlyReti,				/* 0xffdc : IntTX0			*/	//interrupt source:UART0 TX0								
	OnlyReti,				/* 0xffde : IntRX0			*/	//interrupt source:UART0 RX0								
	OnlyReti,				/* 0xffe0 : Reserved		*/									
	OnlyReti,				/* 0xffe2 : Reserved		*/									
	OnlyReti,				/* 0xffe4 : Reserved		*/									
	OnlyReti,				/* 0xffe6 : IntTCA1			*/	//interrupt source:TCA1 16-bit timer								
	OnlyReti,				/* 0xffe8 : IntTCA0			*/	//interrupt source:TCA0 16-bit timer								
	OnlyReti,				/* 0xffea : Reserved		*/									
	OnlyReti,				/* 0xffec : Reserved		*/									
	OnlyReti,			    /* 0xffee : IntRTC			*/	//interrupt source:real-time clock timer								
	OnlyReti,				/* 0xfff0 : IntTBT			*/  //interrupt source:time base timer				
	OnlyReti,				/* 0xfff2 : Reserved		*/									
	OnlyReti,				/* 0xfff4 : IntCFD			*/	//interrupt source:clock fail detection								
	OnlyReti,				/* 0xfff6 : IntLVD			*/	//interrupt source:low voltage detection							
	IntWDT,					/* 0xfff8 : IntWDT			*/	//non-maskable interrupt source:watchdog timer		
	(void *)0xffff,			/* 0xfffa : Reserved		*/									
	IntSWI,					/* 0xfffc : IntSWI/INTUNDEF */	//non-maskable		
	STARTUP					/* 0xfffe : RESET			*/	//non-maskable			
};
#pragma section const
/* ------------------- end of SQ7617 interrupt vector ---------------------- */

